import json
from pathlib import Path
from azureml.pipeline.steps import PythonScriptStep
from azureml.core.runconfig import RunConfiguration

def save_step(
    compute_target,
    environment,
    img_dir,
    save_datastore_name,
    path_on_datastore,
    chunk_index,
    save_datasets,
    name="Save"
):

    run_config = RunConfiguration()
    run_config.target = compute_target
    run_config.environment = environment
    print("Run configuration created for the data saving step")

    step = PythonScriptStep(
        script_name="save.py",
        name=name,
        arguments=[
            "--img_dir",
            img_dir,
            "--chunk_index",
            chunk_index,
            "--save_datastore_name",
            save_datastore_name,
            "--save_datastore_path",
            path_on_datastore,
            "--save_datasets_json",
            json.dumps(save_datasets),
        ],
        inputs=[img_dir],
        compute_target=compute_target,
        runconfig=run_config,
        source_directory=Path(__file__).resolve().parent,
        allow_reuse=True,
    )

    return step



import azureml.core
from azureml.core import Workspace, Datastore, Environment
from azureml.core.compute import ComputeTarget, AmlCompute
from azureml.core.compute_target import ComputeTargetException
from azureml.core.conda_dependencies import CondaDependencies
from azureml.core.runconfig import RunConfiguration


def load_workspace(config_aml_path):
    # Load the workspace from the saved config file
    workspace = Workspace.from_config(config_aml_path)
    print(
        "Ready to use Azure ML {} to work with {}".format(
            azureml.core.VERSION, workspace.name
        )
    )
    return workspace


def get_datastore(workspace, name=""):
    if name == "":
        # Get the default datastore
        datastore = workspace.get_default_datastore()
        print("Default datastore - {} - loaded.".format(datastore.name))
    else:
        datastore = Datastore.get(workspace, name)
        print("Datastore - {} - loaded.".format(datastore.name))
    return datastore


def create_cluster(
    workspace,
    cluster_name="cluster-cpu",
    vm_size="STANDARD_DS3_V2",
    max_nodes=4,
    vnet_resourcegroup_name="",
    vnet_name="",
    subnet_name=""
):
    cluster = None
    try:
        # Check for existing compute target
        cluster = ComputeTarget(workspace=workspace, name=cluster_name)
        print("Found existing cluster, use it.")
        print("cluster_name: {}, vm_size: {}".format(cluster.name, cluster.vm_size))
    except ComputeTargetException:
        # If it doesn't already exist, create it
        try:
            print(
                "Creating cluster: {}, vm_size: {}, max_nodes: {}".format(
                    cluster_name, vm_size, max_nodes
                )
            )
            compute_config = AmlCompute.provisioning_configuration(
                vm_size=vm_size,
                max_nodes=max_nodes,
                vnet_resourcegroup_name=vnet_resourcegroup_name,
                vnet_name=vnet_name,
                subnet_name=subnet_name
            )
            cluster = ComputeTarget.create(workspace, cluster_name, compute_config)
            cluster.wait_for_completion(
                show_output=True
            )
            print("Compute cluster created !")
        except Exception as ex:
            print(ex)
    return cluster


def create_environment(
    workspace,
    environment_name,
    version=None,
    conda_dependencies_file="./aml/conda_dependencies.yml",
    register=True,
    dockerfile=None,
):

    conda_dependencies_file_path = conda_dependencies_file

    # Create a Python environment for the experiment
    environment = Environment(environment_name)

    # Create a set of package dependencies
    aml_packages = CondaDependencies(conda_dependencies_file_path)

    # Add the dependencies to the environment
    environment.python.conda_dependencies = aml_packages

    if dockerfile:
        environment.docker.base_image = None
        DOCKER_PATH = dockerfile
        with open(DOCKER_PATH, "r") as f:
            dockerfile = f.read()
        environment.docker.base_dockerfile = dockerfile
        environment.docker.enabled = True

    if register:
        # Register the environment
        environment.register(workspace=workspace)
        environment = Environment.get(workspace, environment_name, version=version)

    return environment


def create_run_config(cluster, registered_environment):

    # create a new run configuration
    run_config = RunConfiguration()

    # Use the compute you created above.
    run_config.target = cluster

    # Assign the environment to the run configuration
    run_config.environment = registered_environment

    print("Run configuration created.")

    return run_config



import json
from .environment import (
    load_workspace,
    get_datastore,
    create_cluster,
    create_environment,
)
from .pipelines.train_pipeline import train_pipeline
from azureml.core import Experiment

def run_ml_cli_pipeline(args, config_path, ml_cli_template_path):
    # Build Information (when running from AzureDevOps)
    build_id = args.build_id
    build_tags_string = args.build_tags
    config_aml_path = args.config_aml_path
    skip_train_execution = args.skip_train_execution

    build_tags = {}
    if build_tags_string != "":
        build_tags = json.loads(build_tags_string)

    # #################### LOAD CONFIGURATION #####################

    with open(config_path) as config_file:
        config = json.load(config_file)

    with open(ml_cli_template_path) as mlcli_file:
        mlcli_template = mlcli_file.read()

    # Experiment
    experiment_name = config["experiment_name"]

    # #################### ENVIRONMENT CREATION #####################
    # Load the workspace
    workspace = load_workspace(config_aml_path)

    # Load a datastore
    datastore = get_datastore(workspace, config["datastore_name"])

    # Create or get compute clusters
    cfg_cluster_cpu = config["cluster"]["cluster_cpu"]
    cluster_cpu = create_cluster(
        workspace,
        cluster_name=cfg_cluster_cpu["cluster_name"],
        vm_size=cfg_cluster_cpu["vm_size"],
        max_nodes=cfg_cluster_cpu["max_nodes"],
        vnet_resourcegroup_name=cfg_cluster_cpu["vnet_resourcegroup_name"],
        vnet_name=cfg_cluster_cpu["vnet_name"],
        subnet_name=cfg_cluster_cpu["subnet_name"],
    )
    clusters = [cluster_cpu]

    # Create and register Python environments for the experiment
    cfg_light_env = config["environment"]["light_env"]
    light_env = create_environment(
        workspace,
        cfg_light_env["env_name"],
        cfg_light_env["version"],
        cfg_light_env["conda_dependencies_file"],
    )
    cfg_ml_cli_env = config["environment"]["ml-cli_env"]
    ml_cli_env = create_environment(
        workspace,
        cfg_ml_cli_env["env_name"],
        cfg_ml_cli_env["version"],
        cfg_ml_cli_env["conda_dependencies_file"],
        dockerfile=cfg_ml_cli_env["dockerfile"]
    )
    environments = [light_env, ml_cli_env]

    dataset_dict = config["datasets"]

    # #################### PIPELINE CREATION AND SUBMISSION #####################

    # Create the pipeline
    pipeline = train_pipeline(
        workspace, datastore, clusters, environments, dataset_dict, mlcli_template
    )

    if skip_train_execution is False:
        experiment = Experiment(workspace=workspace, name=experiment_name)

        # Run the pipeline
        pipeline_run = experiment.submit(
            pipeline, regenerate_outputs=False, tags=build_tags
        )
        print("Pipeline submitted for execution.", pipeline_run.id)
    else:
        pipeline.validate()

        published_pipeline = pipeline.publish(
            name=experiment_name,
            description="Model training/retraining pipeline",
            version=build_id,
        )

        build_data = {
            "pipeline_id": published_pipeline.id,
            "experiment_name": experiment_name,
            "model_name": "",
        }

        print(f"Published pipeline name: {published_pipeline.name}")
        print(f"Published pipeline id: {published_pipeline.id}")
        print(f"for build {published_pipeline.version}")
        # Save the Pipeline Info for other AzDO jobs after script is complete
        with open("pipeline_info.json", "w+") as out_file:
            out_file.write(json.dumps(build_data))


from pathlib import Path
import json
import numpy as np


def build_agregation(dir_path:str):
    flist = [p for p in Path(dir_path).iterdir() if p.is_file()]

    http600 = []
    http500 = []
    http400 = []
    http200 = []
    ticks = []

    number_retry = 0

    for file_path in flist:
        if not str(file_path).endswith(".json"):
            continue
        with open(file_path, encoding='utf8') as f:
            data = json.load(f)
            number_retry = number_retry + data["TryNumber"]
            status_code = data["StatusCode"]
            ticks.append(data["TicksAt"]/10000000)
            if status_code >= 600:
                p = Path(data["FileDirectory"])
                http500.append(p.stem)
            elif status_code >= 500:
                p = Path(data["FileDirectory"])
                http500.append(p.stem)
            elif status_code >= 400:
                p = Path(data["FileDirectory"])
                http400.append(p.stem)
            elif status_code == 200:
                http200.append(p.stem)


    startAt = np.min(ticks)
    endAt = np.max(ticks)

    result = {
        "total": len(flist),
        "number_retry": number_retry,
        "number200": len(http200),
        "number500": len(http500),
        "number600": len(http600),
        "number400": len(http400),
        "http600": http600,
        "http500": http500,
        "http400": http400,
        "elapsed_time_seconds": int(endAt-startAt)
    }
    return result

def create_summary(dir_path:str, output_file_path:str):
    result = build_agregation(dir_path)
    print(result)
    with open(output_file_path, "w") as out_file:
        json.dump(result, out_file, indent=6)


